@@ -121,11 +121,16 @@ def main1(prog_name, input_config_fn, logger_config_fn=None):
         fc_run_logger.exception('Failed to parse config "{}".'.format(input_config_fn))
         raise
     input_fofn_plf = makePypeLocalFile(config['input_fofn'])
+    genome_size = config.get('genome_size')
+    squash = True if 0 < genome_size < 1000000 else False
     wf = PypeProcWatcherWorkflow(job_type=config['job_type'],
             job_queue=config['job_queue'],
             sge_option=config.get('sge_option', ''),
             watcher_type=config['pwatcher_type'],
-            watcher_directory=config['pwatcher_directory'])
+            watcher_directory=config['pwatcher_directory'],
+            use_tmpdir=config.get('use_tmpdir'),
+            squash=squash
+    )
     run(wf, config,
             os.path.abspath(input_config_fn),
             input_fofn_plf=input_fofn_plf,
@@ -257,10 +262,14 @@ def run(wf, config,
             sys.exit(0)
 
         # Produce new FOFN of preads fasta, based on consensus of overlaps.
+        concurrent_jobs = config['cns_concurrent_jobs']
+        wf.max_jobs = concurrent_jobs
+
         scattered_plf = os.path.join(rawread_dir, 'cns-scatter', 'scattered.json')
         make_task = PypeTask(
                 inputs = {
                     'gathered': las_fopfn_plf,
+                    'db': raw_reads_db_plf,
                 },
                 outputs = {
                     'scattered': scattered_plf,
@@ -295,8 +304,6 @@ def run(wf, config,
         task = make_task(pype_tasks.task_report_pre_assembly)
         wf.addTask(task)
 
-        concurrent_jobs = config['cns_concurrent_jobs']
-        wf.max_jobs = concurrent_jobs
         wf.refreshTargets(exitOnFailure=exitOnFailure)
 
 
@@ -338,6 +345,9 @@ def run(wf, config,
 
     preads_nblock = support.get_nblock(fn(preads_db))
     #### run daligner
+    concurrent_jobs = config['ovlp_concurrent_jobs']
+    wf.max_jobs = concurrent_jobs
+
     config['sge_option_da'] = config['sge_option_pda']
 
     scattered_plf = os.path.join(pread_dir, 'daligner-scatter', 'scattered.json')
@@ -401,9 +411,6 @@ def run(wf, config,
     task, las_fofn_plf, las_fopfn_plf = create_merge_gather_task(os.path.join(pread_dir, 'merge-gather'), p_ids_merged_las)
     wf.addTask(task)
 
-    concurrent_jobs = config['ovlp_concurrent_jobs']
-    wf.max_jobs = concurrent_jobs
-
     wf.refreshTargets(exitOnFailure=exitOnFailure)
 
 
@@ -452,3 +459,6 @@ def main(argv=sys.argv):
         help='(Optional)JSON config for standard Python logging module')
     args = parser.parse_args(argv[1:])
     main1(argv[0], args.config, args.logger)
+
+if __name__ == '__main__':
+    main()
